@@ -1,24 +1,23 @@
 """
-Tools for generating forms based on mongoengine Document schemas.
+Tools for generating forms based on MongoEngine Document schemas.
 """
+
 import inspect
 from werkzeug import secure_filename
-from wtforms import fields as f, validators
-from wtforms import Form
+from wtforms import Form, validators, fields as f
 
 from fields import ModelSelectField, ModelSelectMultipleField, ListField
-# from models import ModelForm
 from mongoengine.fields import ReferenceField, IntField
 
-# from flask_superadmin import form
 from flask_superadmin.model import AdminModelConverter as AdminModelConverter_
 
-__all__ = (
-    'model_fields', 'model_form',
-)
+__all__ = ('model_fields', 'model_form')
 
 
 def converts(*args):
+    """ A convenient decorator for the ModelConverter used to mark which
+    method should be used to convert which MongoEngine field.
+    """
     def _inner(func):
         func._converter_for = frozenset(args)
         return func
@@ -26,6 +25,8 @@ def converts(*args):
 
 
 class ModelConverter(object):
+    """ Manages the way MongoEngine fields are converted into WTForms fields
+    """
     def __init__(self, converters=None):
 
         if not converters:
@@ -51,7 +52,12 @@ class ModelConverter(object):
             kwargs.update(field_args)
 
         if field.required:
-            kwargs['validators'].append(validators.Required())
+
+            # Hacky but necessary, since validators.Required doesn't handle 0 properly
+            if isinstance(field, IntField):
+                kwargs['validators'].append(validators.InputRequired())
+            else:
+                kwargs['validators'].append(validators.Required())
         else:
             kwargs['validators'].append(validators.Optional())
 
@@ -120,6 +126,10 @@ class ModelConverter(object):
     def conv_File(self, model, field, kwargs):
         return f.FileField(**kwargs)
 
+    @converts('ImageField')
+    def conv_Image(self, model, field, kwargs):
+        return f.FileField(**kwargs)
+
     @converts('DecimalField')
     def conv_Decimal(self, model, field, kwargs):
         self._number_common(model, field, kwargs)
@@ -193,39 +203,40 @@ class ModelConverter(object):
         return
 
 
-def model_fields(model, only=None, exclude=None, field_args=None,
-                 converter=None):
+def model_fields(model, fields=None, readonly_fields=None, exclude=None,
+                 field_args=None, converter=None):
     """
-    Generate a dictionary of fields for a given Django model.
+    Generate a dictionary of WTForms fields for a given MongoEngine model.
 
     See `model_form` docstring for description of parameters.
     """
     from mongoengine.base import BaseDocument
     if BaseDocument not in inspect.getmro(model):
-        raise TypeError('model must be a mongoengine Document schema')
+        raise TypeError('Model must be a MongoEngine Document schema')
+
+    readonly_fields = readonly_fields or []
+    exclude = exclude or []
 
     converter = converter or ModelConverter()
     field_args = field_args or {}
 
-    if hasattr(model, '_fields_order'):
-        field_names = model._fields_order
-    else:
-        field_names = model._fields.keys()
-
-    if only:
-        field_names = (x for x in field_names if x in only)
-    elif exclude:
-        field_names = (x for x in field_names if x not in exclude)
+    field_names = fields if fields else model._fields.keys()
+    field_names = (x for x in field_names if x not in exclude)
 
     field_dict = {}
     for name in field_names:
-        model_field = model._fields[name]
-        field = converter.convert(model, model_field, field_args.get(name))
-        if field is not None:
-            field_dict[name] = field
+        if name not in readonly_fields and name not in model._fields:
+            raise KeyError('"%s" is not read-only and does not appear to be a field on the document.' % name)
+
+        if name in model._fields and name not in readonly_fields:
+            model_field = model._fields[name]
+            field = converter.convert(model, model_field, field_args.get(name))
+            if field is not None:
+                field_dict[name] = field
 
     return field_dict
 
+
 import mongoengine.fields as fields
 
 _unset_value = object()
@@ -242,13 +253,15 @@ def data_to_field(field, data):
         return l
     elif isinstance(field, (fields.FileField)):
         if data.filename:
-            gfs = field.proxy_class()
+            gfs = field.proxy_class(
+                        collection_name=field.collection_name,
+                        instance=field.owner_document(),
+                        key=field.name)
             gfs.put(data.stream, filename=secure_filename(data.filename), content_type=data.mimetype)
             return gfs
         elif data.clear:
             return _remove_file_value
         return _unset_value
-        # print '**********',field, data, type(data)
     elif isinstance(field, (fields.ReferenceField, fields.ObjectIdField)) and \
                     isinstance(data, basestring):
         from bson.objectid import ObjectId
@@ -265,28 +278,29 @@ def data_to_document(document, data):
         field_value = data_to_field(field, value)
         if field_value != _unset_value:
             if field_value == _remove_file_value:
-                getattr(new,name).delete()
+                getattr(new, name).delete()
             else:
                 setattr(new, name, field_value)
     return new
 
 
-def model_form(model, base_class=Form, only=None, exclude=None,
-               field_args=None, converter=None):
+def model_form(model, base_class=Form, fields=None, readonly_fields=None,
+               exclude=None, field_args=None, converter=None):
     """
-    Create a wtforms Form for a given mongoengine Document schema::
+    Create a wtforms Form for a given MongoEngine Document schema::
 
         from flaskext.mongoengine.wtf import model_form
         from myproject.myapp.schemas import Article
         ArticleForm = model_form(Article)
 
     :param model:
-        A mongoengine Document schema class
+        A MongoEngine Document schema class
     :param base_class:
         Base form class to extend from. Must be a ``wtforms.Form`` subclass.
-    :param only:
-        An optional iterable with the property names that should be included in
-        the form. Only these properties will have fields.
+    :param fields:
+        An optional iterable with the property names that should be included
+        in the form. Only these properties will have fields. It also
+        determines the order of the fields.
     :param exclude:
         An optional iterable with the property names that should be excluded
         from the form. All other properties will have fields.
@@ -297,20 +311,18 @@ def model_form(model, base_class=Form, only=None, exclude=None,
         A converter to generate the fields based on the model properties. If
         not set, ``ModelConverter`` is used.
     """
-    field_dict = model_fields(model, only, exclude, field_args, converter)
+    field_dict = model_fields(model, fields, readonly_fields, exclude,
+                              field_args, converter)
     field_dict['model_class'] = model
-    # if base_class == ModelForm: base_class = object
 
     def populate_obj(self, obj):
-        # print '_______', self.data, type(self.file_image), '*', self.file_image.file, type(self) #,request, request.files['file_image']
         return data_to_document(obj, self.data)
 
     field_dict['populate_obj'] = populate_obj
 
     return type(model.__name__ + 'Form', (base_class,), field_dict)
 
-    # return type(model.__name__ + 'Form', (ModelForm, base_class,), field_dict)
-
 
 class AdminModelConverter(AdminModelConverter_, ModelConverter):
     pass
+
