@@ -1,10 +1,9 @@
-from sqlalchemy.sql.expression import desc, literal
-from sqlalchemy import schema
-from wtforms.ext.sqlalchemy.orm import model_form
+from sqlalchemy.sql.expression import desc, literal_column
+
+from orm import model_form, AdminModelConverter
 
 from flask_superadmin.model.base import BaseModelAdmin
-
-from .orm import AdminModelConverter
+from sqlalchemy import schema
 
 
 class ModelAdmin(BaseModelAdmin):
@@ -72,30 +71,41 @@ class ModelAdmin(BaseModelAdmin):
         return instance
 
     def delete_models(self, *pks):
-        self.get_objects(*pks).delete(synchronize_session='fetch')
+        objs = self.get_objects(*pks)
+        objs.delete(synchronize_session='fetch')
         self.session.commit()
-        #for obj in self.get_objects(*pks): obj.delete()
         return True
 
-    def construct_search(self, field_name):
-        if field_name.startswith('^'):
-            return literal(field_name[1:]).startswith
-        elif field_name.startswith('='):
-            return literal(field_name[1:]).op('=')
+    def construct_search(self, field_name, op=None):
+        if op == '^':
+            return literal_column(field_name).startswith
+        elif op == '=':
+            return literal_column(field_name).op('==')
         else:
-            return literal(field_name).contains
+            return literal_column(field_name).contains
+
+    def apply_search(self, qs, search_query):
+        or_queries = []
+        # treat spaces as if they were OR operators
+        for word in search_query.split():
+            op = word[:1]
+            if op in ['^', '=']:
+                word = word[1:]
+            orm_lookups = [self.construct_search(str(model_field), op)
+                           for model_field in self.search_fields]
+            or_queries.extend([orm_lookup(word) for orm_lookup in orm_lookups])
+        if or_queries:
+            qs = qs.filter(sum(or_queries))
+        return qs
 
     def get_list(self, page=0, sort=None, sort_desc=None, execute=False, search_query=None):
         qs = self.get_queryset()
 
         # Filter by search query
         if search_query and self.search_fields:
-            orm_lookups = [self.construct_search(str(search_field))
-                           for search_field in self.search_fields]
-            for bit in search_query.split():
-                or_queries = [orn_lookup(bit) for orm_lookup in orm_lookups]
-                qs = qs.filter(sum(or_queries))
+            qs = self.apply_search(qs, search_query)
 
+        #Calculate number of rows
         count = qs.count()
 
         #Order queryset
@@ -114,4 +124,3 @@ class ModelAdmin(BaseModelAdmin):
             qs = qs.all()
 
         return count, qs
-
