@@ -6,20 +6,20 @@ from flask import request, url_for, redirect, flash, abort
 
 from flask_superadmin.babel import gettext
 from flask_superadmin.base import BaseView, expose
-# from flask_superadmin.form import format_form
-from flask_superadmin.form import BaseForm, ChosenSelectWidget, DatePickerWidget, \
-    DateTimePickerWidget, FileField
+from flask_superadmin.form import (BaseForm, ChosenSelectWidget, FileField,
+                                   DatePickerWidget, DateTimePickerWidget)
 
 import traceback
 
+
 class AdminModelConverter(object):
     def convert(self, *args, **kwargs):
         field = super(AdminModelConverter, self).convert(*args, **kwargs)
         if field:
             widget = field.kwargs.get('widget', field.field_class.widget)
-            # print field, widget
             if isinstance(widget, widgets.Select):
-                field.kwargs['widget'] = ChosenSelectWidget(multiple=widget.multiple)
+                field.kwargs['widget'] = ChosenSelectWidget(
+                    multiple=widget.multiple)
             elif issubclass(field.field_class, fields.DateTimeField):
                 field.kwargs['widget'] = DateTimePickerWidget()
             elif issubclass(field.field_class, fields.DateField):
@@ -30,6 +30,8 @@ class AdminModelConverter(object):
 
 
 first_cap_re = re.compile('(.)([A-Z][a-z]+)')
+
+
 def camelcase_to_space(name):
     return first_cap_re.sub(r'\1 \2', name)
 
@@ -39,19 +41,29 @@ def prettify(str):
 
 
 class BaseModelAdmin(BaseView):
+    """ BaseModelAdmin provides create/edit/delete functionality for an
+    abstract Model. The abstraction is further customized by the
+    backend-specific model admin (see flask_superadmin/model/backends/) and
+    by the user-defined admin classes inheriting from ModelAdmin.
     """
-    BaseModelView provides create/edit/delete functionality for a peewee Model.
-    """
-    # columns to display in the list index - can be field names or callables on
-    # a model instance, though in the latter case they will not be sortable
+
+    # Number of objects to display per page in the list view
     list_per_page = 20
+
+    # Columns to display in the list index - can be field names or callables.
+    # Admin's methods have higher priority than the fields/methods on
+    # the model or document.
     list_display = tuple()
 
-    # form parameters, lists of fields
-    exclude = None
-    only = None
-    fields = None
-    readonly_fields = []
+    # Only fields with names specified in `fields` will be displayed in the
+    # form (minus the ones mentioned in `exclude`). The order is preserved,
+    # too. You can also include methods that are on the model admin, or on the
+    # model/document, as long as they are marked as read-only (i.e. included
+    # in `readonly_fields`). Priority of fields' lookup: methods on the model
+    # admin, methods/fields on the model/document.
+    fields = tuple()
+    readonly_fields = tuple()
+    exclude = tuple()
 
     form = None
 
@@ -64,10 +76,14 @@ class BaseModelAdmin(BaseView):
     add_template = 'admin/model/add.html'
     delete_template = 'admin/model/delete.html'
 
-    search_fields = None
-    actions = None
+    search_fields = tuple()
 
     field_overrides = {}
+
+    # A dictionary of field_name: overridden_params_dict, e.g.
+    #   { 'name': { 'label': 'Name', 'description': 'This is a name' } }
+    # Parameters that can be overridden: label, description, validators,
+    # filters, default
     field_args = None
 
     @staticmethod
@@ -95,43 +111,70 @@ class BaseModelAdmin(BaseView):
 
     def get_column(self, instance, name):
         parts = name.split('.')
-        obj = instance
+        value = instance
         for p in parts:
-            obj = getattr(obj, p, None)
-            if not obj:
-                break
-        return self.get_column_value(obj)
 
-    def get_column_value(self, value):
-        if callable(value):
-            return value()
+            # admin's methods have higher priority than the fields/methods on
+            # the model or document. If a callable is found on the admin
+            # level, it's also passed an instance object
+            if hasattr(self, p) and callable(getattr(self, p)):
+                value = getattr(self, p)(instance)
+            else:
+                value = getattr(value, p, None)
+                if callable(value):
+                    value = value()
+
+            if not value:
+                break
+
         return value
 
     def get_reference(self, column_value):
         for model, model_view in self.admin._models:
             if type(column_value) == model:
-                return '/admin/%s/%s/' % (model_view.endpoint, column_value.pk)
+                return '/admin/%s/%s/' % (model_view.endpoint,
+                                          self.get_pk(column_value))
+
+    def get_readonly_fields(self, instance):
+        ret_vals = {}
+        for field in self.readonly_fields:
+            self_field = getattr(self, field, None)
+            if callable(self_field):
+                val = self_field(instance)
+            else:
+                val = getattr(instance, field)
+                if callable(val):
+                    val = val()
+            if not isinstance(val, dict):
+                # Check if the value is a reference field to a doc/model
+                # registered in the admin. If so, link to it.
+                reference = self.get_reference(val)
+                val = {
+                    'label': prettify(field),
+                    'value': val,
+                    'url': reference if reference else None
+                }
+            ret_vals[field] = val
+        return ret_vals
 
     def get_converter(self):
         raise NotImplemented()
 
-    def get_form(self, include_readonly=False):
-        if include_readonly:
-            exclude = self.exclude
-        else:
-            exclude = list(set(self.exclude or []) | set(self.readonly_fields or []))
-
-        only = list(set(self.only or []) - set(exclude or []))
+    def get_model_form(self):
+        """ Returns the model form, should get overridden in backend-specific
+        view.
+        """
+        raise NotImplemented()
 
+    def get_form(self):
         model_form = self.get_model_form()
         converter = self.get_converter()
         if isinstance(converter, type):
             converter = converter()
-        form = model_form(self.model, base_class=BaseForm, only=only,
-                          exclude=exclude, field_args=self.field_args,
+        form = model_form(self.model, base_class=BaseForm, fields=self.fields,
+                          readonly_fields=self.readonly_fields,
+                          exclude=self.exclude, field_args=self.field_args,
                           converter=converter)
-
-        form.readonly_fields = self.readonly_fields
         return form
 
     def get_add_form(self):
@@ -188,18 +231,21 @@ class BaseModelAdmin(BaseView):
 
     @expose('/add/', methods=('GET', 'POST'))
     def add(self):
+        if not self.can_create:
+            abort(403)
+
         Form = self.get_add_form()
         if request.method == 'POST':
             form = Form()
             if form.validate_on_submit():
                 try:
-                    instance = self.save_model(self.model(), form, True)
+                    instance = self.save_model(self.model(), form, adding=True)
                     flash(gettext('New %(model)s saved successfully',
                           model=self.get_display_name()), 'success')
                     return self.dispatch_save_redirect(instance)
                 except Exception, ex:
                     print traceback.format_exc()
-                    if self.session:
+                    if hasattr(self, 'session'):
                         self.session.rollback()
                     flash(gettext('Failed to add model. %(error)s',
                           error=str(ex)), 'error')
@@ -237,7 +283,8 @@ class BaseModelAdmin(BaseView):
             sort = '-' + sort
         if page == 0:
             page = None
-        return url_for(self.get_url_name('index'), page=page, sort=sort, q=search_query)
+        return url_for(self.get_url_name('index'), page=page, sort=sort,
+                       q=search_query)
 
     def sort_url(self, sort, desc=None):
         if sort and desc:
@@ -253,8 +300,8 @@ class BaseModelAdmin(BaseView):
         # Grab parameters from URL
         if request.method == 'POST':
             id_list = request.form.getlist('_selected_action')
-            if id_list and (request.form.get('action-delete') or \
-                request.form.get('action', None) == 'delete'):
+            if id_list and (request.form.get('action-delete') or
+                            request.form.get('action', None) == 'delete'):
                 return self.delete(*id_list)
 
         sort, sort_desc = self.sort
@@ -275,50 +322,50 @@ class BaseModelAdmin(BaseView):
         except self.model.DoesNotExist:
             abort(404)
 
-        Form = self.get_form(include_readonly=request.method == 'GET')
+        Form = self.get_form()
 
         if request.method == 'POST':
             form = Form(obj=instance)
             if form.validate_on_submit():
                 try:
-                    self.save_model(instance, form, False)
-                    flash('Changes to %s saved successfully' % self.get_display_name(),
-                          'success')
+                    self.save_model(instance, form, adding=False)
+                    flash(
+                        'Changes to %s saved successfully' % self.get_display_name(),
+                        'success'
+                    )
                     return self.dispatch_save_redirect(instance)
                 except Exception, ex:
                     print traceback.format_exc()
-                    flash(gettext('Failed to edit model. %(error)s', error=str(ex)),
-                          'error')
+                    flash(gettext('Failed to edit model. %(error)s',
+                                  error=str(ex)), 'error')
         else:
             form = Form(obj=instance)
 
         return self.render(self.edit_template, model=self.model, form=form,
                            pk=self.get_pk(instance), instance=instance)
 
-    @expose('/<pk>/delete', methods=('GET', 'POST'))
+    @expose('/<pk>/delete/', methods=('GET', 'POST'))
     def delete(self, pk=None, *pks):
+        if not self.can_delete:
+            abort(403)
+
         if pk:
             pks += pk,
-            # collected = {}
-            # if self.delete_collect_objects:
-            #     for obj in query:
-            #         collected[obj.get_pk()] = self.collect_objects(obj)
 
         if request.method == 'POST' and 'confirm_delete' in request.form:
             count = len(pks)
             self.delete_models(*pks)
 
-            flash('Successfully deleted %s %ss' % (count, self.get_display_name()),
-                  'success')
+            flash(
+                'Successfully deleted %s %ss' % (count, self.get_display_name()),
+                'success'
+            )
             return redirect(url_for(self.get_url_name('index')))
         else:
             instances = self.get_objects(*pks)
 
-        return self.render(self.delete_template,
-            instances=instances
-            # query=query,
-            # collected=collected,
-        )
+        return self.render(self.delete_template, instances=instances)
 
 
-class ModelAdmin(object): pass
+class ModelAdmin(BaseModelAdmin):
+    pass
